/*******************************************************************
* TESTFUNC.C - This file contains all of the test functions used   *
* in the article "Loop optimization techniques" in IMC May 1991.   *
*                                                                  *
* There are 5 sets of test functions, corresponding to the test    *
* cases discussed in the article. By uncommenting one of the       *
* #define statements below you can choose which set of test        *
* functions to run. For example, to test the code from Figures 1a  *
* and 1b, you would uncomment the line: "#define FIGURE1 0"        *
*******************************************************************/

#include <string.h>          /* memcpy(), memset()                */

/*-------- Select test functions by uncommenting ONE of these ----*/
#define FIGURE1 0
//#define MEMSET  0
//#define FIGURE2 0
//#define FIGURE3 0
//#define FIGURE4 0

/*-------- Global variables used by test functions ---------------*/
char a = 3, b = 5, c = 7;
char values[100];
char buf1[100], buf2[100];
int table1[100], table2[100], table3[100];

/*******************************************************************
* This empty function is used to determine the standard function   *
* call overhead.                                                   *
*******************************************************************/
void emptyfunc(void)
    {
    }

/*******************************************************************
* These functions test the code shown in Figures 1a and 1b         *
*******************************************************************/
#ifdef FIGURE1
void testfunc1( void )
    {
    int counter = 0;

    for ( ; counter<100; counter++)
        buf1[counter] = buf2[counter];
    }

void testfunc2( void )
    {
    memcpy(buf1, buf2, 100);
    }
#endif

/*******************************************************************
* These functions test the memset() function against a for loop    *
*******************************************************************/
#ifdef MEMSET
void testfunc1( void )
    {
    int counter = 0;

    for ( ; counter<100; counter++)
        buf1[counter] = 0;
    }

void testfunc2( void )
    {
    memset(buf1, 0, 100);
    }
#endif

/*******************************************************************
* These functions test the code shown in Figures 2a and 2b         *
*******************************************************************/
#ifdef FIGURE2
void testfunc1( void )
    {
    int counter = 0;

    for ( ; counter<100; counter++)
        values[counter] = counter;
    }

void testfunc2( void )
{
    register int counter = 0;

    for ( ; counter<100; counter++)
        values[counter] = counter;
}
#endif

/*******************************************************************
* These functions test the code shown in Figures 3a and 3b         *
*******************************************************************/
#ifdef FIGURE3
void testfunc1( void )
    {
    int counter = 0;

    for ( ; counter<100; counter++)
        values[counter] = counter + a + (b * c);
    }

void testfunc2( void )
    {
    int counter = 0;
    int t = a + (b * c);

    for ( ; counter<100; counter++)
        values[counter] = counter + t;
    }
#endif

/*******************************************************************
* These functions test the code shown in Figures 4a and 4b         *
*******************************************************************/
#ifdef FIGURE4
void testfunc1( void )
    {
    int counter = 0;

    for ( ; counter<100; counter++) {
        table1[counter] = a + (counter * 10);
        table2[counter] = b + (counter * 10);
        table3[counter] = c + (counter * 10);
        }
    }

void testfunc2( void )
    {
    int counter = 0, t = 0;

    for ( ; counter<100; counter++) {
        table1[counter] = a + t;
        table2[counter] = b + t;
        table3[counter] = c + t;
        t += 10;
        }
    }
#endif
