/********************************************************************
*	VGAFADE.C - This program draws 15 strings in colors 0 through	 *
* 		15 and then fades the strings in and out by	changing the 	 *
*		RGB values of the colors in the VGA DAC registers. 			 *
*                                                                   *
*	To compile: "cl vgafade.c"										 *
********************************************************************/

#pragma comment (lib, "GRAPHICS.LIB")	/* Library-search record	*/

#include <stdio.h>		/* printf()								  	*/
#include <conio.h>		/* kbhit(), getch()							*/
#include <dos.h>		/* int86(), int86x() 						*/
#include <graph.h>		/* _setvideomode(), _clearscreen(), ...		*/

									/* Macros to get SEG:OFS values */
#define GET_SEG(a) ((unsigned)((long)(void _far *)(a)>>16))
#define GET_OFF(a) ((unsigned)((void _far *)(a)))

#define X_OFFSET	27				/* used to center text in screen*/
#define Y_OFFSET	 5				/* used to center text in screen*/

union REGS regs;					/* Registers for int86() calls	*/
struct SREGS sregs;				/* Segment regs. for int86x()	*/
unsigned char storeRGB[16][3];		/* DAC register storage		   	*/
unsigned char storeMAP[17];		/* Palette register storage		*/

/*******************************************************************
* SetDACreg() - Set the RGB values of one DAC register				*
*******************************************************************/
void SetDACreg(int reg, int R, int G, int B)
	{
	regs.x.ax = 0x1010;				/* Set specific DAC register	*/
	regs.x.bx = reg;				/* Color register				*/
	regs.h.dh = (char)R;			/* Set red value				*/
	regs.h.ch = (char)G;			/* Set green value				*/
	regs.h.cl = (char)B;			/* Set blue value				*/
	int86(0x10,&regs,&regs);		/* Int0x10 Func0x10 Subfunc0x10	*/
	}

/*******************************************************************
* main() - main body of program VGAFADE.C	 						*
*******************************************************************/
void main(void)
	{
	int  i, j;						/* Counters						*/
	char str[40];					/* Work string					*/

	_setvideomode(_DEFAULTMODE);	/* Initialize MSC graphics		*/
	_displaycursor(_GCURSOROFF);	/* Turn off cursor				*/
	_clearscreen(_GCLEARSCREEN);	/* Clear the screen				*/

	/*----------Save current VGA color palette----------------------*/
	regs.x.ax = 0x1009;				/* Read all palette registers	*/
	sregs.es  = GET_SEG(storeMAP);	/* Load segment of table in ES	*/
	regs.x.dx = GET_OFF(storeMAP);	/* Load offset of table in DX	*/
	int86x(0x10, &regs, &regs, &sregs); /* Func 0x10, subfunc 0x15	*/

	/*----------Save RGB values for the 16 palette colors-----------*/
	for (i=0; i<16; i++)
		{
		regs.x.ax = 0x1015;			/* Get RGB vals from DAC table	*/
		regs.x.bx = storeMAP[i];	/* ... for color number 'i'		*/
		int86(0x10,&regs,&regs);	/* Call func 0x10, subfunc 0x15	*/
		storeRGB[i][0] = regs.h.dh;	/* Save red value				*/
		storeRGB[i][1] = regs.h.ch;	/* Save green value				*/
		storeRGB[i][2] = regs.h.cl;	/* Save blue value				*/
		}

	/*----------Draw 15 lines of text in 15 colors------------------*/
	for (i=1; i<16; i++)
		{							/* first, position text			*/
		_settextposition(i+Y_OFFSET, X_OFFSET);
		_settextcolor(i);			/* then, choose color (1 to 16)	*/
		sprintf(str,				/* next, build text string		*/
			"This is text color #%d", i);
		_outtext(str);				/* finally, draw text			*/
		}

	/*----------Fade text in and out one line at a time-------------*/
	j = 0;							/* Initialize j prior to loop	*/
	while ( !kbhit() )				/* Keep looping till key pressed*/
		{
		if (++j > 15)				/* Move to next color in palette*/
			j = 1;					/* If at end, start over with 1	*/

		for (i=0; i<30000; i++)		/* Delay briefly before fading	*/
			;
									/* Point arrow at current color */
		_settextposition(j+Y_OFFSET, strlen(str)+X_OFFSET+2);
		_outtext("<--");			/* Draw arrow					*/

									/* Fade out color 'j' out in 64 */
		for (i=64; i>=0; i--)		/* 	uniform steps.				*/
			SetDACreg(storeMAP[j], (i*storeRGB[j][0])/64,
						(i*storeRGB[j][1])/64, (i*storeRGB[j][2])/64);

									/* Fade color 'j' back in with  */
		for (i=0; i<=64; i++)		/*   64 uniform steps.		    */
			SetDACreg(storeMAP[j],(i*storeRGB[j][0])/64,
						(i*storeRGB[j][1])/64, (i*storeRGB[j][2])/64);

									/* Position back over arrow		*/
		_settextposition(j+Y_OFFSET, strlen(str)+X_OFFSET+2);
		_outtext("   ");			/* Erase arrow					*/
		}

	/*----------Restore original RGB values in DAC table------------*/
	for (i=0; i<16; i++)
		{
		regs.x.ax=0x1010;			/* Set RGB values	in DAC table*/
		regs.x.bx=storeMAP[i];		/* ... for color number 'i'		*/
		regs.h.dh=storeRGB[i][0];	/* Restore red value			*/
		regs.h.ch=storeRGB[i][1];	/* Restore green value			*/
		regs.h.cl=storeRGB[i][2];	/* Restore blue value			*/
		int86(0x10,&regs,&regs);	/* Int0x10 func0x10 subfunc0x10 */
		}

	_displaycursor(_GCURSORON);		/* Turn cursor back on			*/
	_clearscreen(_GCLEARSCREEN);	/* Clear the screen				*/
	}
